"""
This module contains RuleCondition implementation.

There is no need to import this module directly. Use the methods of
the L{Ruleset object <ruleset.Ruleset>} to create and manipulate
RuleConditions.

>>> rs.rulecondition_add(rs.RULECONDITION_AND, "tesla")
"""

from thisversion import PLDB
import plutil

_Prop_RuleCondition = plutil.Prop('id', 'name', 'creator', 'items', 'creation_date', 'operator', 'modification_date', 'modified_by')

def _add(pldb, op, name):
	cid = pldb.session.ruleconditions_add(name, op)
	id, tstamp = plutil._do(pldb, cid, plutil._AddCB(pldb, pldb.session.ruleconditions_add_handle))

	res = RuleCondition(id = id,
			    name = name,
			    creator = pldb.user,
			    items = [],
			    creation_date = tstamp,
			    operator = op,
			    modification_date = tstamp,
			    modified_by = pldb.user)
	res._pldb = pldb
	return res

def _list(pldb):
	cid = pldb.session.ruleconditions_list()
	return plutil._do(pldb, cid, plutil._ListCB(pldb, pldb.session.ruleconditions_list_handle, RuleCondition))


class RuleCondition(object):

	__slots__ = ('_prop', '_pldb')

	def __init__(self, _pldb=None, **props):
		self._pldb = _pldb
		self._prop = _Prop_RuleCondition(**props)

	def __repr__(self):
		return "<PacketLogic RuleCondition '%i' '%s'>" % (self._prop.id, self._prop.name)

	def rename(self, name):
		"""
		Renames rulecondition to given name
		@type name: C{str}
		@param name: New name
		"""
		cid = self._pldb.session.ruleconditions_rename(self._prop.id, name)
		plutil._do(self._pldb, cid, plutil._AddCB(self._pldb, self._pldb.session.ruleconditions_rename_handle))
		self._prop.name = name
		self._pldb._rulecondition_invalidate_cache()

	def set_items(self, items):
		"""
		Set items on rulecondition as list of (conditiontype, object id)
			>>> rulecondition.set_items([(0,285)]
			>>> rulecondition.items
			[(0, 285)]

		@type items: C{list}
		@param items: C{dict} of C{int}:C{int}
		"""
		cid = self._pldb.session.ruleconditions_set_items(self._prop.id, items)
		plutil._do(self._pldb, cid, plutil._AddCB(self._pldb, self._pldb.session.ruleconditions_set_items_handle))
		self._prop.items = items

	def remove(self):
		"""
		Removes this rulecondition from pldb
		"""
		cid = self._pldb.session.ruleconditions_remove(self._prop.id)
		plutil._do(self._pldb, cid, plutil._Callback(self._pldb))
		self._pldb._rulecondition_invalidate_cache()

	class Item(dict):
		pass

	id = property(lambda self: self._prop.id, doc="id number of this rulecondition")
	name = property(lambda self: self._prop.name, doc="name of rulecondition")
	creator = property(lambda self: self._prop.creator, doc="creator of rulecondition")
	items = property(lambda self: self._prop.items, doc="Items")
	creation_date = property(lambda self: self._prop.creation_date, doc="creation date of rulecondition")
	operator = property(lambda self: self._prop.operator, doc="Operator")
	modification_date = property(lambda self: self._prop.modification_date, doc="latest modification of rulecondition")
	modified_by = property(lambda self: self._prop.modified_by, doc="latest modifier of rulecondition")
